"Main interface for dynamodb service Client"
from __future__ import annotations

from datetime import datetime
import sys
from typing import Any, Dict, List, overload
from botocore.client import BaseClient
from botocore.exceptions import ClientError as Boto3ClientError

# pylint: disable=import-self
import mypy_boto3_dynamodb.client as client_scope

# pylint: disable=import-self
import mypy_boto3_dynamodb.paginator as paginator_scope
from mypy_boto3_dynamodb.type_defs import (
    AttributeDefinitionTypeDef,
    AttributeValueTypeDef,
    AttributeValueUpdateTypeDef,
    AutoScalingSettingsUpdateTypeDef,
    BatchGetItemOutputTypeDef,
    BatchWriteItemOutputTypeDef,
    ConditionTypeDef,
    CreateBackupOutputTypeDef,
    CreateGlobalTableOutputTypeDef,
    CreateTableOutputTypeDef,
    DeleteBackupOutputTypeDef,
    DeleteItemOutputTypeDef,
    DeleteTableOutputTypeDef,
    DescribeBackupOutputTypeDef,
    DescribeContinuousBackupsOutputTypeDef,
    DescribeContributorInsightsOutputTypeDef,
    DescribeEndpointsResponseTypeDef,
    DescribeGlobalTableOutputTypeDef,
    DescribeGlobalTableSettingsOutputTypeDef,
    DescribeLimitsOutputTypeDef,
    DescribeTableOutputTypeDef,
    DescribeTableReplicaAutoScalingOutputTypeDef,
    DescribeTimeToLiveOutputTypeDef,
    ExpectedAttributeValueTypeDef,
    GetItemOutputTypeDef,
    GlobalSecondaryIndexAutoScalingUpdateTypeDef,
    GlobalSecondaryIndexTypeDef,
    GlobalSecondaryIndexUpdateTypeDef,
    GlobalTableGlobalSecondaryIndexSettingsUpdateTypeDef,
    KeySchemaElementTypeDef,
    KeysAndAttributesTypeDef,
    ListBackupsOutputTypeDef,
    ListContributorInsightsOutputTypeDef,
    ListGlobalTablesOutputTypeDef,
    ListTablesOutputTypeDef,
    ListTagsOfResourceOutputTypeDef,
    LocalSecondaryIndexTypeDef,
    PointInTimeRecoverySpecificationTypeDef,
    ProvisionedThroughputTypeDef,
    PutItemOutputTypeDef,
    QueryOutputTypeDef,
    ReplicaAutoScalingUpdateTypeDef,
    ReplicaSettingsUpdateTypeDef,
    ReplicaTypeDef,
    ReplicaUpdateTypeDef,
    ReplicationGroupUpdateTypeDef,
    RestoreTableFromBackupOutputTypeDef,
    RestoreTableToPointInTimeOutputTypeDef,
    SSESpecificationTypeDef,
    ScanOutputTypeDef,
    StreamSpecificationTypeDef,
    TagTypeDef,
    TimeToLiveSpecificationTypeDef,
    TransactGetItemTypeDef,
    TransactGetItemsOutputTypeDef,
    TransactWriteItemTypeDef,
    TransactWriteItemsOutputTypeDef,
    UpdateContinuousBackupsOutputTypeDef,
    UpdateContributorInsightsOutputTypeDef,
    UpdateGlobalTableOutputTypeDef,
    UpdateGlobalTableSettingsOutputTypeDef,
    UpdateItemOutputTypeDef,
    UpdateTableOutputTypeDef,
    UpdateTableReplicaAutoScalingOutputTypeDef,
    UpdateTimeToLiveOutputTypeDef,
    WriteRequestTypeDef,
)

# pylint: disable=import-self
import mypy_boto3_dynamodb.waiter as waiter_scope

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal


__all__ = ("Client",)


class Client(BaseClient):
    """
    [DynamoDB.Client documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client)
    """

    exceptions: client_scope.Exceptions

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def batch_get_item(
        self,
        RequestItems: Dict[str, KeysAndAttributesTypeDef],
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
    ) -> BatchGetItemOutputTypeDef:
        """
        [Client.batch_get_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.batch_get_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def batch_write_item(
        self,
        RequestItems: Dict[str, List[WriteRequestTypeDef]],
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ReturnItemCollectionMetrics: Literal["SIZE", "NONE"] = None,
    ) -> BatchWriteItemOutputTypeDef:
        """
        [Client.batch_write_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.batch_write_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def can_paginate(self, operation_name: str) -> bool:
        """
        [Client.can_paginate documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.can_paginate)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def create_backup(self, TableName: str, BackupName: str) -> CreateBackupOutputTypeDef:
        """
        [Client.create_backup documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.create_backup)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def create_global_table(
        self, GlobalTableName: str, ReplicationGroup: List[ReplicaTypeDef]
    ) -> CreateGlobalTableOutputTypeDef:
        """
        [Client.create_global_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.create_global_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def create_table(
        self,
        AttributeDefinitions: List[AttributeDefinitionTypeDef],
        TableName: str,
        KeySchema: List[KeySchemaElementTypeDef],
        LocalSecondaryIndexes: List[LocalSecondaryIndexTypeDef] = None,
        GlobalSecondaryIndexes: List[GlobalSecondaryIndexTypeDef] = None,
        BillingMode: Literal["PROVISIONED", "PAY_PER_REQUEST"] = None,
        ProvisionedThroughput: ProvisionedThroughputTypeDef = None,
        StreamSpecification: StreamSpecificationTypeDef = None,
        SSESpecification: SSESpecificationTypeDef = None,
        Tags: List[TagTypeDef] = None,
    ) -> CreateTableOutputTypeDef:
        """
        [Client.create_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.create_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def delete_backup(self, BackupArn: str) -> DeleteBackupOutputTypeDef:
        """
        [Client.delete_backup documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.delete_backup)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def delete_item(
        self,
        TableName: str,
        Key: Dict[str, AttributeValueTypeDef],
        Expected: Dict[str, ExpectedAttributeValueTypeDef] = None,
        ConditionalOperator: Literal["AND", "OR"] = None,
        ReturnValues: Literal["NONE", "ALL_OLD", "UPDATED_OLD", "ALL_NEW", "UPDATED_NEW"] = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ReturnItemCollectionMetrics: Literal["SIZE", "NONE"] = None,
        ConditionExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
        ExpressionAttributeValues: Dict[str, AttributeValueTypeDef] = None,
    ) -> DeleteItemOutputTypeDef:
        """
        [Client.delete_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.delete_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def delete_table(self, TableName: str) -> DeleteTableOutputTypeDef:
        """
        [Client.delete_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.delete_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_backup(self, BackupArn: str) -> DescribeBackupOutputTypeDef:
        """
        [Client.describe_backup documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_backup)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_continuous_backups(self, TableName: str) -> DescribeContinuousBackupsOutputTypeDef:
        """
        [Client.describe_continuous_backups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_continuous_backups)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_contributor_insights(
        self, TableName: str, IndexName: str = None
    ) -> DescribeContributorInsightsOutputTypeDef:
        """
        [Client.describe_contributor_insights documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_contributor_insights)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_endpoints(self) -> DescribeEndpointsResponseTypeDef:
        """
        [Client.describe_endpoints documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_endpoints)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_global_table(self, GlobalTableName: str) -> DescribeGlobalTableOutputTypeDef:
        """
        [Client.describe_global_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_global_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_global_table_settings(
        self, GlobalTableName: str
    ) -> DescribeGlobalTableSettingsOutputTypeDef:
        """
        [Client.describe_global_table_settings documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_global_table_settings)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_limits(self) -> DescribeLimitsOutputTypeDef:
        """
        [Client.describe_limits documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_limits)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_table(self, TableName: str) -> DescribeTableOutputTypeDef:
        """
        [Client.describe_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_table_replica_auto_scaling(
        self, TableName: str
    ) -> DescribeTableReplicaAutoScalingOutputTypeDef:
        """
        [Client.describe_table_replica_auto_scaling documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_table_replica_auto_scaling)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_time_to_live(self, TableName: str) -> DescribeTimeToLiveOutputTypeDef:
        """
        [Client.describe_time_to_live documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.describe_time_to_live)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Dict[str, Any] = None,
        ExpiresIn: int = 3600,
        HttpMethod: str = None,
    ) -> None:
        """
        [Client.generate_presigned_url documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.generate_presigned_url)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_item(
        self,
        TableName: str,
        Key: Dict[str, AttributeValueTypeDef],
        AttributesToGet: List[str] = None,
        ConsistentRead: bool = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ProjectionExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
    ) -> GetItemOutputTypeDef:
        """
        [Client.get_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.get_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_backups(
        self,
        TableName: str = None,
        Limit: int = None,
        TimeRangeLowerBound: datetime = None,
        TimeRangeUpperBound: datetime = None,
        ExclusiveStartBackupArn: str = None,
        BackupType: Literal["USER", "SYSTEM", "AWS_BACKUP", "ALL"] = None,
    ) -> ListBackupsOutputTypeDef:
        """
        [Client.list_backups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.list_backups)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_contributor_insights(
        self, TableName: str = None, NextToken: str = None, MaxResults: int = None
    ) -> ListContributorInsightsOutputTypeDef:
        """
        [Client.list_contributor_insights documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.list_contributor_insights)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_global_tables(
        self, ExclusiveStartGlobalTableName: str = None, Limit: int = None, RegionName: str = None
    ) -> ListGlobalTablesOutputTypeDef:
        """
        [Client.list_global_tables documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.list_global_tables)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_tables(
        self, ExclusiveStartTableName: str = None, Limit: int = None
    ) -> ListTablesOutputTypeDef:
        """
        [Client.list_tables documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.list_tables)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_tags_of_resource(
        self, ResourceArn: str, NextToken: str = None
    ) -> ListTagsOfResourceOutputTypeDef:
        """
        [Client.list_tags_of_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.list_tags_of_resource)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def put_item(
        self,
        TableName: str,
        Item: Dict[str, AttributeValueTypeDef],
        Expected: Dict[str, ExpectedAttributeValueTypeDef] = None,
        ReturnValues: Literal["NONE", "ALL_OLD", "UPDATED_OLD", "ALL_NEW", "UPDATED_NEW"] = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ReturnItemCollectionMetrics: Literal["SIZE", "NONE"] = None,
        ConditionalOperator: Literal["AND", "OR"] = None,
        ConditionExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
        ExpressionAttributeValues: Dict[str, AttributeValueTypeDef] = None,
    ) -> PutItemOutputTypeDef:
        """
        [Client.put_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.put_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def query(
        self,
        TableName: str,
        IndexName: str = None,
        Select: Literal[
            "ALL_ATTRIBUTES", "ALL_PROJECTED_ATTRIBUTES", "SPECIFIC_ATTRIBUTES", "COUNT"
        ] = None,
        AttributesToGet: List[str] = None,
        Limit: int = None,
        ConsistentRead: bool = None,
        KeyConditions: Dict[str, ConditionTypeDef] = None,
        QueryFilter: Dict[str, ConditionTypeDef] = None,
        ConditionalOperator: Literal["AND", "OR"] = None,
        ScanIndexForward: bool = None,
        ExclusiveStartKey: Dict[str, AttributeValueTypeDef] = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ProjectionExpression: str = None,
        FilterExpression: str = None,
        KeyConditionExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
        ExpressionAttributeValues: Dict[str, AttributeValueTypeDef] = None,
    ) -> QueryOutputTypeDef:
        """
        [Client.query documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.query)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def restore_table_from_backup(
        self,
        TargetTableName: str,
        BackupArn: str,
        BillingModeOverride: Literal["PROVISIONED", "PAY_PER_REQUEST"] = None,
        GlobalSecondaryIndexOverride: List[GlobalSecondaryIndexTypeDef] = None,
        LocalSecondaryIndexOverride: List[LocalSecondaryIndexTypeDef] = None,
        ProvisionedThroughputOverride: ProvisionedThroughputTypeDef = None,
    ) -> RestoreTableFromBackupOutputTypeDef:
        """
        [Client.restore_table_from_backup documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.restore_table_from_backup)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def restore_table_to_point_in_time(
        self,
        SourceTableName: str,
        TargetTableName: str,
        UseLatestRestorableTime: bool = None,
        RestoreDateTime: datetime = None,
        BillingModeOverride: Literal["PROVISIONED", "PAY_PER_REQUEST"] = None,
        GlobalSecondaryIndexOverride: List[GlobalSecondaryIndexTypeDef] = None,
        LocalSecondaryIndexOverride: List[LocalSecondaryIndexTypeDef] = None,
        ProvisionedThroughputOverride: ProvisionedThroughputTypeDef = None,
    ) -> RestoreTableToPointInTimeOutputTypeDef:
        """
        [Client.restore_table_to_point_in_time documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.restore_table_to_point_in_time)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def scan(
        self,
        TableName: str,
        IndexName: str = None,
        AttributesToGet: List[str] = None,
        Limit: int = None,
        Select: Literal[
            "ALL_ATTRIBUTES", "ALL_PROJECTED_ATTRIBUTES", "SPECIFIC_ATTRIBUTES", "COUNT"
        ] = None,
        ScanFilter: Dict[str, ConditionTypeDef] = None,
        ConditionalOperator: Literal["AND", "OR"] = None,
        ExclusiveStartKey: Dict[str, AttributeValueTypeDef] = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        TotalSegments: int = None,
        Segment: int = None,
        ProjectionExpression: str = None,
        FilterExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
        ExpressionAttributeValues: Dict[str, AttributeValueTypeDef] = None,
        ConsistentRead: bool = None,
    ) -> ScanOutputTypeDef:
        """
        [Client.scan documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.scan)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def tag_resource(self, ResourceArn: str, Tags: List[TagTypeDef]) -> None:
        """
        [Client.tag_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.tag_resource)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def transact_get_items(
        self,
        TransactItems: List[TransactGetItemTypeDef],
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
    ) -> TransactGetItemsOutputTypeDef:
        """
        [Client.transact_get_items documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.transact_get_items)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def transact_write_items(
        self,
        TransactItems: List[TransactWriteItemTypeDef],
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ReturnItemCollectionMetrics: Literal["SIZE", "NONE"] = None,
        ClientRequestToken: str = None,
    ) -> TransactWriteItemsOutputTypeDef:
        """
        [Client.transact_write_items documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.transact_write_items)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def untag_resource(self, ResourceArn: str, TagKeys: List[str]) -> None:
        """
        [Client.untag_resource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.untag_resource)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_continuous_backups(
        self,
        TableName: str,
        PointInTimeRecoverySpecification: PointInTimeRecoverySpecificationTypeDef,
    ) -> UpdateContinuousBackupsOutputTypeDef:
        """
        [Client.update_continuous_backups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_continuous_backups)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_contributor_insights(
        self,
        TableName: str,
        ContributorInsightsAction: Literal["ENABLE", "DISABLE"],
        IndexName: str = None,
    ) -> UpdateContributorInsightsOutputTypeDef:
        """
        [Client.update_contributor_insights documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_contributor_insights)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_global_table(
        self, GlobalTableName: str, ReplicaUpdates: List[ReplicaUpdateTypeDef]
    ) -> UpdateGlobalTableOutputTypeDef:
        """
        [Client.update_global_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_global_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_global_table_settings(
        self,
        GlobalTableName: str,
        GlobalTableBillingMode: Literal["PROVISIONED", "PAY_PER_REQUEST"] = None,
        GlobalTableProvisionedWriteCapacityUnits: int = None,
        GlobalTableProvisionedWriteCapacityAutoScalingSettingsUpdate: AutoScalingSettingsUpdateTypeDef = None,
        GlobalTableGlobalSecondaryIndexSettingsUpdate: List[
            GlobalTableGlobalSecondaryIndexSettingsUpdateTypeDef
        ] = None,
        ReplicaSettingsUpdate: List[ReplicaSettingsUpdateTypeDef] = None,
    ) -> UpdateGlobalTableSettingsOutputTypeDef:
        """
        [Client.update_global_table_settings documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_global_table_settings)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_item(
        self,
        TableName: str,
        Key: Dict[str, AttributeValueTypeDef],
        AttributeUpdates: Dict[str, AttributeValueUpdateTypeDef] = None,
        Expected: Dict[str, ExpectedAttributeValueTypeDef] = None,
        ConditionalOperator: Literal["AND", "OR"] = None,
        ReturnValues: Literal["NONE", "ALL_OLD", "UPDATED_OLD", "ALL_NEW", "UPDATED_NEW"] = None,
        ReturnConsumedCapacity: Literal["INDEXES", "TOTAL", "NONE"] = None,
        ReturnItemCollectionMetrics: Literal["SIZE", "NONE"] = None,
        UpdateExpression: str = None,
        ConditionExpression: str = None,
        ExpressionAttributeNames: Dict[str, str] = None,
        ExpressionAttributeValues: Dict[str, AttributeValueTypeDef] = None,
    ) -> UpdateItemOutputTypeDef:
        """
        [Client.update_item documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_item)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_table(
        self,
        TableName: str,
        AttributeDefinitions: List[AttributeDefinitionTypeDef] = None,
        BillingMode: Literal["PROVISIONED", "PAY_PER_REQUEST"] = None,
        ProvisionedThroughput: ProvisionedThroughputTypeDef = None,
        GlobalSecondaryIndexUpdates: List[GlobalSecondaryIndexUpdateTypeDef] = None,
        StreamSpecification: StreamSpecificationTypeDef = None,
        SSESpecification: SSESpecificationTypeDef = None,
        ReplicaUpdates: List[ReplicationGroupUpdateTypeDef] = None,
    ) -> UpdateTableOutputTypeDef:
        """
        [Client.update_table documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_table)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_table_replica_auto_scaling(
        self,
        TableName: str,
        GlobalSecondaryIndexUpdates: List[GlobalSecondaryIndexAutoScalingUpdateTypeDef] = None,
        ProvisionedWriteCapacityAutoScalingUpdate: AutoScalingSettingsUpdateTypeDef = None,
        ReplicaUpdates: List[ReplicaAutoScalingUpdateTypeDef] = None,
    ) -> UpdateTableReplicaAutoScalingOutputTypeDef:
        """
        [Client.update_table_replica_auto_scaling documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_table_replica_auto_scaling)
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def update_time_to_live(
        self, TableName: str, TimeToLiveSpecification: TimeToLiveSpecificationTypeDef
    ) -> UpdateTimeToLiveOutputTypeDef:
        """
        [Client.update_time_to_live documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Client.update_time_to_live)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_backups"]
    ) -> paginator_scope.ListBackupsPaginator:
        """
        [Paginator.ListBackups documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Paginator.ListBackups)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_tables"]
    ) -> paginator_scope.ListTablesPaginator:
        """
        [Paginator.ListTables documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Paginator.ListTables)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_tags_of_resource"]
    ) -> paginator_scope.ListTagsOfResourcePaginator:
        """
        [Paginator.ListTagsOfResource documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Paginator.ListTagsOfResource)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(self, operation_name: Literal["query"]) -> paginator_scope.QueryPaginator:
        """
        [Paginator.Query documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Paginator.Query)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(self, operation_name: Literal["scan"]) -> paginator_scope.ScanPaginator:
        """
        [Paginator.Scan documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Paginator.Scan)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_waiter(self, waiter_name: Literal["table_exists"]) -> waiter_scope.TableExistsWaiter:
        """
        [Waiter.TableExists documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Waiter.TableExists)
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_waiter(
        self, waiter_name: Literal["table_not_exists"]
    ) -> waiter_scope.TableNotExistsWaiter:
        """
        [Waiter.TableNotExists documentation](https://boto3.amazonaws.com/v1/documentation/api/1.10.35/reference/services/dynamodb.html#DynamoDB.Waiter.TableNotExists)
        """


class Exceptions:
    BackupInUseException: Boto3ClientError
    BackupNotFoundException: Boto3ClientError
    ClientError: Boto3ClientError
    ConditionalCheckFailedException: Boto3ClientError
    ContinuousBackupsUnavailableException: Boto3ClientError
    GlobalTableAlreadyExistsException: Boto3ClientError
    GlobalTableNotFoundException: Boto3ClientError
    IdempotentParameterMismatchException: Boto3ClientError
    IndexNotFoundException: Boto3ClientError
    InternalServerError: Boto3ClientError
    InvalidRestoreTimeException: Boto3ClientError
    ItemCollectionSizeLimitExceededException: Boto3ClientError
    LimitExceededException: Boto3ClientError
    PointInTimeRecoveryUnavailableException: Boto3ClientError
    ProvisionedThroughputExceededException: Boto3ClientError
    ReplicaAlreadyExistsException: Boto3ClientError
    ReplicaNotFoundException: Boto3ClientError
    RequestLimitExceeded: Boto3ClientError
    ResourceInUseException: Boto3ClientError
    ResourceNotFoundException: Boto3ClientError
    TableAlreadyExistsException: Boto3ClientError
    TableInUseException: Boto3ClientError
    TableNotFoundException: Boto3ClientError
    TransactionCanceledException: Boto3ClientError
    TransactionConflictException: Boto3ClientError
    TransactionInProgressException: Boto3ClientError
